/***************************************************************************
                          basetable.h  -  dtfarmer
                             -------------------
    begin                : Mon Sep 11 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BASETABLE_H
#define BASETABLE_H
#include <stdlib.h>

class Node;

/**This class is the base for all tables.
  *@author Siegfried Nijssen
  */

class BaseTable {
private:
  Node *node;
public: 
  BaseTable();
    /* set current node is used to set the node which is currently
     * being considered belonging with this table, this allows
     * derivate tables to lookup the node and its additionaldata,
     * for which eg. the truth value is computed; this could be necessary
     * if the test is not completely filled in yet.
     * -- Node is not a parameter of the truth value checking functions
     *    to maintain code compatibility with the functions which are used
     *    for association rule farmer --
     * This node is used most of the times by most of the functions in this
     * class.
     */
  void setCurrentNode ( Node *node ) { this->node = node; }
  Node *getCurrentNode () { return node; }
  virtual ~BaseTable();
    /* this function is called just before the node quality is evaluated,
     * and should be used to fill 'additionaldata' of the node with information
     * for the final test and rearrange left and right keyfacttables of a
     * decisiontree consequently, if necessary
     */
  virtual void finish () { ; }
    /* this function is called by the constructor of a node, to
     * initialize the 'additionaldata' of the node. The default sets
     * additionaldata to NULL; derived classes can override
     */
  virtual void init ();
    /* this function can be used to clear up information in additionaldata
     * when the node is removed
     */
  virtual void remove () { ; }
    /* the keyfacttables have place for additional data for each example
     * separately. This function returns the information which is to be placed
     * in this location for this kind of table. Optionally uses a node to
     * determine the contents.
     * the default returns NULL.
     */
  virtual void *determineAdditionalData () { return NULL; }
    /* some nodes contain additional information, which could be
     * printed
     */
  virtual double additionalPunishment () { return 0.0; }
  virtual char *string () { return ""; }
};

#endif
