/***************************************************************************
                          keyfacttable.cpp  -  dtfarmer
                             -------------------
    begin                : Mon Aug 7 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "output.h"
#include "keyfacttable.h"
#include "globaldata.h"
#include "type.h"

KeyFactTable::KeyFactTable( int outputdim, bool copy ){
  this->copy = copy;
  this->outputdim = outputdim;
  this->size = 0;
  first = last = NULL;
}

void KeyFactTable::setCopy ( bool copy ) {
  this->copy = copy;
}

void KeyFactTable::addFact ( INTTYPE *value, void *additionaldata = NULL ) {
  KeyFactTableCell *c = new KeyFactTableCell;
  c->next = NULL;
  c->additionaldata = additionaldata;
  if ( copy ) {
    c->value = new INTTYPE[outputdim];
    for ( int i = 0; i < outputdim; i++ )
      c->value[i] = value[i];
  }
  else
    c->value = value;
  if ( !first )
    last = first = c;
  else {
    last->next = c;
    last = c;
  }
  size++;
}

INTTYPE *KeyFactTable::popFact () {
  if ( first ) {
    KeyFactTableCell *kftc = first;
    INTTYPE *value = kftc->value;
    first = kftc->next;
    if ( !first )
      last = NULL;
    delete kftc;
    size--;
    return value;
  }
  else
    return NULL;
}

void KeyFactTable::append ( KeyFactTable *kft ) {
  if ( !kft )
    return;
  if ( first )
    last->next = kft->first;
  else
    first = kft->first;
  last = kft->last;
  size += kft->size;
  kft->first = kft->last = NULL;
  kft->size = 0;
}

void KeyFactTable::shuffle () {
  KeyFactTableCell *c1, *c2;

  if ( size > 2 ) {
    for ( int k = 0, i; k < crossvalidation; k++ )
      for ( i = 0, c1 = first; i < size - k * crossvalidation; i++, c1 = c1->next ) {
        if ( rand () < RAND_MAX / crossvalidation ) {
          c2 = c1->next;
          c1->next = c1->next->next;
          last->next = c2;
          c2->next = NULL;
          last = c2;
        }
      }
  }
}

KeyFactTable::~KeyFactTable(){
  KeyFactTableCell *help = first;

  while ( first ) {
    first = first->next;
    if ( copy )
      delete[] help->value;
    delete help;
    help = first;
  }
}

void KeyFactTable::iterate ( Iterator iterator, void *extraparams ) {
  KeyFactTableCell *help = first;

  while ( help ) {
    iterator ( help->value, help->additionaldata, extraparams );
    help = help->next;
  }
}
