/***************************************************************************
                          keyfacttables.cpp  -  dtfarmer
                             -------------------
    begin                : Tue Aug 8 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "keyfacttables.h"
#include "globaldata.h"
#include "type.h"
#include <math.h>

void *KeyFactTables::sad;
KeyFactTables::Iterator KeyFactTables::itp;
int KeyFactTables::cnr;

KeyFactTables::KeyFactTables ( int size ):
  _impurity ( -1.0 ),
  _nrkeyfacts ( -1 ) {
  this->size = size;
  additionaldata = new void*[size];
  keyfacttable = new KeyFactTable*[size];

  for ( int i = 0; i < size; i++ )
    additionaldata[i] = keyfacttable[i] = NULL;
}

KeyFactTables::~KeyFactTables () {
  for ( int i = 0; i < size; i++ )
    if ( keyfacttable[i] )
      delete keyfacttable[i];

  delete[] additionaldata;
  delete[] keyfacttable;
}

void KeyFactTables::it ( INTTYPE *value, void *additionaldata, void *extraparams ) {
  KeyFactTables::itp ( KeyFactTables::cnr, KeyFactTables::sad, value, additionaldata, extraparams );
}

void KeyFactTables::iterate ( Iterator iterator, void *extraparams ) {
  itp = iterator;
  for ( int i = 0; i < size; i++ ) {
    cnr = i;
    if ( keyfacttable[i] ) {
      sad = additionaldata[i];
      keyfacttable[i]->iterate ( it, extraparams );
    }
  }
}

double KeyFactTables::impurity () {

  if ( _impurity < 0.0 ) {
    _impurity = 0;
    _nrkeyfacts = 0;
    int s;

    for ( int i = 0; i < size; i++ ) {
      if ( keyfacttable[i] )
        s = keyfacttable[i]->getSize ();
      else
        s = 0;
      if ( s )
        _impurity += - s * log ( s );
      _nrkeyfacts += s;
    }

    if ( _nrkeyfacts )
      _impurity = _impurity / _nrkeyfacts + log ( _nrkeyfacts );
    else
      _impurity = 0;
  }

  return _impurity;
}

int KeyFactTables::nrkeyfacts () {
  if ( _nrkeyfacts == -1 ) {
    int s;
    _nrkeyfacts = 0;
    for ( int i = 0; i < size; i++ ) {
      if ( keyfacttable[i] )
        s = keyfacttable[i]->getSize ();
      else
        s = 0;
      _nrkeyfacts += s;
    }
  }
  return _nrkeyfacts;
}

void KeyFactTables::append ( KeyFactTables *kft ) {
  for ( int i = 0; i < size; i++ )
    if ( keyfacttable[i] )
      keyfacttable[i]->append ( kft->keyfacttable[i] );
    else {
      keyfacttable[i] = kft->keyfacttable[i];
      kft->keyfacttable[i] = NULL;
    }
  _nrkeyfacts = -1;
  _impurity = -1.0;
  delete kft;
}

KeyFactTables *KeyFactTables::extractSample ( int fraction ) {
  KeyFactTables *nkft = new KeyFactTables ( size );
  int i, j, is, outputdim = akeypredicate->size;
  _nrkeyfacts = -1;
  _impurity = -1.0;

  for ( i = 0; i < size; i++ ) {
    is = ( keyfacttable[i]->getSize () + fraction - 1 ) / fraction;
    if ( is > 0 )
      nkft->keyfacttable[i] = new KeyFactTable ( outputdim, false );
    for ( j = 0; j < is; j++ )
      nkft->keyfacttable[i]->addFact ( keyfacttable[i]->popFact () );
    if ( is > 0 )
      nkft->keyfacttable[i]->setCopy ( true );
  }

  return nkft;
}

KeyFactTables *KeyFactTables::extractSampleOnAttribute ( int nr ) {
  KeyFactTables *nkft = new KeyFactTables ( size );
  INTTYPE *value;
  int t;

  int i, j, is, outputdim = akeypredicate->size;
  _nrkeyfacts = -1;
  _impurity = -1.0;

  for ( i = 0; i < size; i++ ) {
    is = keyfacttable[i]->getSize ();
    t = 0;
    for ( j = 0; j < is; j++ ) {
      value = keyfacttable[i]->popFact ();
      if ( value[0] == nr ) {
        if ( !nkft->keyfacttable[i] )
          nkft->keyfacttable[i] = new KeyFactTable ( outputdim, false );
        nkft->keyfacttable[i]->addFact ( value );
        t++;
      }
      else
        keyfacttable[i]->addFact ( value );
    }
    if ( t )
      nkft->keyfacttable[i]->setCopy ( true );
  }

  return nkft;
}

bool KeyFactTables::isLookaheadFraction ( KeyFactTables *kfts ) {
  int s1, s2;
  for ( int i = 0; i < size; i++ ) {
    s1 = kfts->keyfacttable[i] ? kfts->keyfacttable[i]->getSize () : 0;
    s2 = keyfacttable[i] ? keyfacttable[i]->getSize () : 0;
    if ( s1 / lookaheadfraction < s2 )
      return false;
  }
  return true;
}

void KeyFactTables::shuffle () {
  for ( int i = 0; i < size; i++ )
    keyfacttable[i]->shuffle ();
}