/***************************************************************************
                          keyfacttables.h  -  dtfarmer
                             -------------------
    begin                : Tue Aug 8 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KEYFACTTABLES_H
#define KEYFACTTABLES_H
#include "keyfacttable.h"
#include "type.h"

/**Contains a predefined number of KeyFactTables together
with additional information.
  *@author Siegfried Nijssen
  */

class KeyFactTables {
  public:
    typedef void (*Iterator) ( int cnr, void *additionaldata_class, INTTYPE *value, void *additionaldata_value, void *extraparams );
    KeyFactTables ( int size );
    ~KeyFactTables ();
    KeyFactTables *extractSample ( int fraction );
    KeyFactTables *extractSampleOnAttribute ( int nr );
    void shuffle ();
    void append ( KeyFactTables *kft ); // also deletes kft
    void iterate ( Iterator iterator, void *extraparams = NULL );
    void **additionaldata;
    KeyFactTable **keyfacttable;
    double impurity ();
    int nrkeyfacts ();
    int size; // size is the number of classes!
    bool isLookaheadFraction ( KeyFactTables *kfts );
  private:
    double _impurity;   // impurity, computed by computeStats
    int _nrkeyfacts; // total number of key facts, computed by computeStats
                     // value -1 means, not initialised. computeStats
	 	    // will use this to determine whether a computation
		    // is really needed
    static void it ( INTTYPE *value, void *additionaldata, void *extraparams );
    static void *sad;
    static Iterator itp;
    static int cnr;
};

#endif
