/***************************************************************************
                          main.cpp  -  dtfarmer
                             -------------------
    begin                : Mon Aug  7 21:55:30 CEST 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "parser.h"
#include "interpreter.h"
#include "globaldata.h"
#include "modefacttable.h" // to be removed
#include "tree_global.h"
#include "extern.h"
#include <iostream.h>
#include <sys/time.h>

void header () {
  cerr << "************** " << endl
       << "** dtFarmer ** " << endl
       << "************** " << endl
       << "Discovery of first order decision trees" << endl
       << "Siegfried Nijssen 2000" << endl;
}

int main(int argc, char *argv[])
{
  char *filename;
  timeval tv1, tv2;
  bool validate = ( argc == 3 && !strcmp ( argv[1], "-v" ) );

  header ();

  // remove argument
  if ( validate ) {
    argv[1] = argv[2];
    argc--;
    cerr << "-- Validation only" << endl;
  }

  if ( argc != 2 ) {
    cerr << "No input file!" << endl;
    return -1;
  }
  filename = argv[1];
  parse ( filename, interpret, 'g' );
  cerr << "[Finished reading input]" << endl;
  assertfilledbias ();
  modetable.print ();
  cerr << "[Extern initialization]" << endl;
  extern_init ();
  globaldata_print ();

  if ( validate ) {
    double t = 0.0, v, t2;
    int tt = 0;
    cerr << "[Starting validation]" << endl;
    if ( validationtable )
      validateInitValidationTable ();

    if ( shuffle )
      keyfacttables->shuffle ();
    train_keyfacttables = keyfacttables;

    for ( int i = 0; i < crossvalidation; i++ ) {
      if ( usefirstattributeforvalidation )
        test_keyfacttables = train_keyfacttables->extractSampleOnAttribute ( i );
      else
        test_keyfacttables = train_keyfacttables->extractSample ( crossvalidation );

      gettimeofday ( &tv1, NULL );

      createTree ();

      gettimeofday ( &tv2, NULL );
      tt += ( tv2.tv_sec - tv1.tv_sec ) * 1000 + ( tv2.tv_usec - tv1.tv_usec ) / 1000;

      v = validateTree ();

      removeTree ();

      cerr << i << ": correctly classified: " << v << "% (" << v * test_keyfacttables->nrkeyfacts () / 100 << ")" << endl;
      t += v;
      t2 += v * test_keyfacttables->nrkeyfacts () / 100;

      train_keyfacttables->append ( test_keyfacttables );
    }
    cerr << "Total correctly classified: " << t / crossvalidation << "% (" << t2 << ")" << endl;

    cerr << "[Average searchtime: " << tt / crossvalidation << "ms]" << endl;
    if ( validationtable )
      validateRemoveValidationTable ();
  }
  else {
    test_keyfacttables = train_keyfacttables = keyfacttables;
    cerr << "[Starting search]" << endl;
    gettimeofday ( &tv1, NULL );

    createTree ();

    gettimeofday ( &tv2, NULL );
    cerr << "Testing recall: " << validateTree () << "% correct" << endl;
    cerr << "[Finished building in " << ( tv2.tv_sec - tv1.tv_sec ) * 1000 + ( tv2.tv_usec - tv1.tv_usec ) / 1000  << "ms]" << endl;
    printTree ();
    removeTree ();
  }
}
