/***************************************************************************
                          modefacttable.cpp  -  description
                             -------------------
    begin                : Tue Apr 11 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "type.h"
#include "modefacttable.h"
#include "typetable.h"
#include "output.h"
#include "globaldata.h"

ModeFactTable::ModeFactTable( TypeTable &typetable, ModeData *modedata ):
  tinputs ( 0 ),
  toutputs ( 0 ) {
  int *inputsizes;
  this->modedata = modedata;
  outputdim = modedata->nroutput;
  inputdim = modedata->size - outputdim;
  inputsizes = new int[inputdim];
  for ( int i = 0, j = 0; i < modedata->size; i++ )
    if ( modedata->params[i].type == Constant || modedata->params[i].type == Plus )
      inputsizes[j++] = typetable.array[modedata->params[i].number]->constnumbers;
  initTable ( cells, inputdim, inputsizes );
}

ModeFactTable::~ModeFactTable() {
  removeTable ( cells, inputdim, inputsizes );
}

void ModeFactTable::initTable ( ModeFactTableCell &cells, int dimensions, int *sizes ) {
  if ( dimensions > 1 ) {
    cells.celldata = new ModeFactTableCell[sizes[0]];
    for ( int i = 0; i < sizes[0]; i++ )
      initTable ( cells.celldata[i], dimensions - 1, sizes + 1 );
  }
  else {
    cells.celldata = new ModeFactTableCell[sizes[0]];
    for ( int i = 0; i < sizes[0]; i++ )
      cells.celldata[i].truecoors = NULL;
  }
}

void ModeFactTable::removeTable ( ModeFactTableCell cells, int dimensions, int *sizes ) {
  ListModeFactCoorsElement *mfc, *temp;
  if ( dimensions > 1 )
    for ( int i = 0; i < sizes[0]; i++ )
      removeTable ( cells.celldata[i], dimensions, sizes + 1 );
  else {
    for ( int i = 0; i < sizes[0]; i++ ) {
      mfc = cells.celldata[i].truecoors;

      while ( mfc ) {
        temp = mfc->next;
        delete mfc;
        mfc = temp;
      }
    }
  }
  delete[] cells.celldata;
}

void ModeFactTable::setFact ( INTTYPE *coor ) {
  ModeFactTableCell *cell = &cells;
  ListModeFactCoorsElement *coorle = new ListModeFactCoorsElement;
  coorle->value = new INTTYPE[outputdim];
  int j = 0, k;
  for ( k = 0; k < modedata->size; k++ )
    if ( IS_OUTPUT_PARAM ( modedata->params + k ) )
      coorle->value[j++] = coor[k];
    else
      cell = cell->celldata + coor[k];
  coorle->next = cell->truecoors;
  cell->truecoors = coorle;
  if ( !coorle->next )
    tinputs++;
  toutputs++;
}

double ModeFactTable::getLookaheadPunishment () {
  if ( !constlapunishment ) {
    if ( tinputs == 0 )
      return -1.0; // an error!
    double val = ((double) toutputs) / tinputs;
    if ( val > 1.0 )
      return ( val - 1.0 ) * lookaheadpunishment + 1.0;
    else
      return 1.0;
  }
  else
    return lookaheadpunishment;
}