/***************************************************************************
                            -  description
                             -------------------
    begin                : Wed Mar 22 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "modetable.h"
#include "modefacttable.h"
#include "floatrangefacttable.h"
#include "predicatetable.h"
#include "globaldata.h"
#include "output.h"
#include "type.h"

ModeTable::ModeTable ():
  keynr ( 1 ),
  size ( 0 ),
  filledarray ( false ) {

}

ModeTable::~ModeTable () {
  if ( filledarray ) {
    for ( int i = 0; i < size; i++ )
      delete[] array[i].params;
    delete[] array;
  }
  else {
    ModeDataLE *walk = list, *temp;
    
    while ( walk ) {
      temp = walk;
      walk = walk->next;
      delete[] temp->data->params;
      delete temp->data;
      delete temp;
    }
  }
}

void ModeTable::fillArray ( PredicateTable &predicatetable ) {
  array = new ModeData[size];
  ModeDataLE *walk = list, *temp;

  for ( int i = 0; i < size; i++ ) {
    walk->data->nr = size - i - 1;
    array[size - i - 1] = *walk->data;
    predicatetable.array[walk->data->predicate]->nmodes++;
    temp = walk;
    walk = walk->next;
//    delete temp->data; // toch ook?
    delete temp;
  }

  filledarray = true;
}

void ModeTable::fillDependencies ( PredicateTable &predicatetable, TypeTable &typetable ) {
  bool modepattern[size], typepattern[typetable.size];
  int i, j, k, nr, *params;
  PredicateData *pd;
  ModeData *md;
  keynr--;
  keyfacttables = new KeyFactTables ( keynr );
  keynrtopnr = new int[keynr];

  for ( i = 0; i < size; i++ ) {
    md = array + i;
    pd = predicatetable.array[md->predicate];

    if ( md->outputs ) {
      for ( j = 0; j < typetable.size; j++ )
	typepattern[j] = false;
      for ( j = 0; j < array[i].size; j++ )
	if ( md->params[j].type == Min || md->params[j].type == MinEx )
          typepattern[md->params[j].number] = true;
      nr = 0;
      for ( j = 0; j < size; j++ ) {
	modepattern[j] = false;
	for ( k = 0; k < array[j].size; k++ )
          if ( array[j].params[k].type == Plus && typepattern[array[j].params[k].number] ) {
            modepattern[j] = true;
            nr++;
            break;
          }
      }
      md->depmodsize = nr;
      md->dependentmodes = new int[nr];
      nr = 0;
      for ( j = 0; j < size; j++ )
	if ( modepattern[j] )
          md->dependentmodes[nr++] = j;

      md->reuses = false;
      md->equalingmode = false;
      if ( md->key ) {
        keyfacttables->keyfacttable[md->key - 1] = md->table.keyfacttable = new KeyFactTable ( md->size, true );
        keyfacttables->additionaldata[md->key - 1] = NULL;
	keynrtopnr[md->key - 1] = pd->nr;
      }
      else {
         // search for a previous mode which might have another constant
	 // as `input' or another output mode, but is the same otherwise, 
	 // and reuse the table associated with that mode.
	BaseModeFactTable *mft = NULL;
	
        for ( j = 0; j < i; j++ )
	  if ( array[j].predicate == array[i].predicate ) {
	    // check parameters
	    for ( k = 0; k < array[i].size; k++ )
	      if (    ( IS_OUTPUT_PARAM ( array[j].params + k ) && IS_INPUT_PARAM ( array[i].params + k ) )
	           || ( IS_INPUT_PARAM ( array[j].params + k ) && IS_OUTPUT_PARAM ( array[i].params + k ) ) )
		break;
	    if ( k == array[j].size ) {
	      mft = array[j].table.modefacttable;
	      break;
	    }
	  }
	if ( !mft )
	  if ( md->functionname )
	    md->table.modefacttable = new ExternalModeFactTable ( md->functionname );
	  else
            md->table.modefacttable = new ModeFactTable ( typetable, md );
	else {
	  md->reuses = true;
	  md->table.modefacttable = mft;
	}
      }
    }
    else {
      md->reuses = false;
      md->equalingmode = false;
      md->dependentmodes = NULL;
      md->depmodsize = 0;
      for ( j = 0; j < size; j++ )
        if ( i != j && array[j].predicate == md->predicate )
          for ( k = 0; k < array[i].size; k++ )
            if ( md->params[k].type == Plus &&
                 IS_OUTPUT_PARAM ( array[j].params + k ) ) {
              md->equalingmode = true;
              goto quitloops;
            }
quitloops:
      params = new int[md->size];
      for ( j = 0; j < md->size; j++ )
        params[j] = typetable.array[pd->params[j]]->constnumbers;
      if ( !pd->facttable )
        if ( md->functionname )
          if ( strcmp ( md->functionname, "rangecheck" ) )
            pd->facttable = new ExternalFactTable ( md->functionname );
          else
            pd->facttable = new FloatRangeFactTable;
	else
          pd->facttable = new FactTable ( md->size, params );
      else
        md->reuses = true;
      md->table.facttable = pd->facttable;
    }
     // put this mode in the list of the predicate
    if ( pd->filln == 0 )
      pd->modes = new ModeData*[pd->nmodes];
    if ( md->reuses )
      pd->nmodes--; // over allocated...
    else
      pd->modes[pd->filln++] = md;
  }
}

ModeData* ModeTable::addMode ( int number, int predicate, int parsize, ModeParamData *params, int idnr, char *functionname, bool key ) {
  bool outputs = false;
  int i, j, k, nrinput = 0, nroutput = 0;

  ModeDataLE *elt = new ModeDataLE;
  elt->next = list;
  elt->data = new ModeData;
  elt->data->number = number;
  elt->data->predicate = predicate;
  elt->data->size = parsize;
  elt->data->params = params;
  elt->data->idnr = idnr;
  if ( functionname ) {
    elt->data->functionname = new char[strlen ( functionname ) + 1];
    strcpy ( elt->data->functionname, functionname );
  }
  else
    elt->data->functionname = NULL;
  list = elt;

  for ( i = 0; i < parsize; i++ )
    if ( params[i].type == Plus )
      nrinput++;
    else
      if ( params[i].type != Constant ) {
        nroutput++;
        outputs = true;
      }

  elt->data->outputs = outputs;
  elt->data->nrinput = nrinput;
  elt->data->inputvarpos = new int[nrinput];
  elt->data->nroutput = nroutput;
  elt->data->outputvarpos = new int[nroutput];

  for ( i = 0, j = 0, k = 0; i < parsize; i++ )
    if ( params[i].type == Plus )
      elt->data->inputvarpos[j++] = i;
    else
      if ( params[i].type != Constant )
        elt->data->outputvarpos[k++] = i;

  size++;

  if ( key )
    elt->data->key = keynr++;
  else
    elt->data->key = 0;

  return elt->data;
}

void ModeTable::print () {
  int i, j, w;

  if ( filledarray ) {
    cout << "+---+---+---+-----------+-----------------+--------------------+" << endl
         << "| N | k | i | predicate | params          | dependent modes    |" << endl
         << "+---+---+---+-----------+-----------------+--------------------+" << endl;
    for ( i = 0; i < size; i++ ) {
      w = 0;
      cout << "|" << setw ( 2 ) << array[i].nr << " |" << setw ( 2 ) << array[i].number << " |" << setw(2) << array[i].idnr << " |" << setw ( 10 ) << array[i].predicate << " | ";
      for ( j = 0; j < array[i].size; j++ ) {
        switch ( array[i].params[j].type ) {
          case Plus: cout << "+"; break;
	  case Min: cout << "-"; break;
	  case MinEx: cout << "\\"; break;
	  case Constant: cout << "#"; break;
	}
	cout << setw ( 2 ) << array[i].params[j].number;
	w += 3;
	if ( j < array[i].size - 1 ) {
	  cout << ", ";
	  w += 2;
	}
      }
      cout << setw ( 18 - w ) << " | ";
      w = 0;
      for ( j = 0; j < array[i].depmodsize; j++ ) {
        cout << setw ( 2 ) << array[i].dependentmodes[j];
        w += 2;
        if ( j < array[i].depmodsize - 1 ) {
          cout << ", ";
          w += 2;
        }
      }
      cout << setw ( 20 - w ) << "|";
      if ( array[i].outputs && array[i].table.modefacttable && !array[i].key )
        cout << " " << array[i].table.modefacttable->getLookaheadPunishment () << endl;
      else
        cout << endl;
    }
    cout << "+---+---+---+-----------+-----------------+--------------------+" << endl;
  }
  else
    cerr << "Error: display not possible for unfilled data." << endl;
}

void setFact ( ModeData *md, INTTYPE *coors ) {
  if ( !md->reuses )
    if ( md->functionname ) {
      ERR << "Error: facts may not be used for external modes!";
      ERR_F
    }
    else
      if ( md->key )
        ((KeyFactTable*) md->table.keyfacttable)->addFact ( coors );
      else
        if ( md->outputs )
          ((ModeFactTable*) md->table.modefacttable)->setFact ( coors );
        else
          ((FactTable*) md->table.facttable)->setFact ( coors, true );
}
