/***************************************************************************
                          tree.h  -  description
                             -------------------
    begin                : Sat Mar 25 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef TREE_H
#define TREE_H
#include "keyfacttables.h"
#include "type.h"

struct Node {
  Node ( short int _mode, int size, INTTYPE *_params, void *_additionaldata = NULL );
  ~Node ();
  Node *left, *right;
  void *additionaldata;
  INTTYPE mode;
  INTTYPE *params;
  bool enabled; // if descendants which need refinement
  KeyFactTables *leftfacts, *rightfacts;
  char leftclass, rightclass;
  void finish_additionaldata ();
  double impurity ( double &ratio );
  double impurity ( int leftnrkeyfacts, double leftimpurity, double &ratio );
  bool isRightLookaheadFraction ( KeyFactTables *kfts ) {
    return rightfacts->isLookaheadFraction ( kfts );
  }
  char *toString ();
  unsigned int *classn; // the number of examples in this node for each class,
                     // maintained for displaying purposes
};

class NodeList {
  private:
    struct NodeListElt {
      Node *node;
      NodeListElt *next, *prev;
    };
  public:
    NodeList ( bool del = true );
    ~NodeList ();
    void addNode ( Node *node );
    void append ( NodeList *nodelist );
      // appends and empties the given nodelist
    Node *first () { walkelt = firstelt; return walkelt ? walkelt->node : NULL; }
    Node *next () { walkelt = walkelt->next; return walkelt ? walkelt->node: NULL; }
    Node *current () { return walkelt->node; }
    Node *remove (); // removes from the list, does not kill the node
                     // even if del = true
                     // returns the next node after the removed, if any
  private:
    NodeListElt *firstelt, *lastelt, *walkelt;
    bool del;
};

#define MODE(node) ((node)->mode)

void initTree ();
void removeTree ();
void printTree ();
bool expandTree ();
void createTree ();

#endif
