/***************************************************************************
                          tree_validation.cpp  -  dtfarmer
                             -------------------
    begin                : Mon Aug 28 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include "tree_global.h"
#include "type.h"

int correctclasn, incorrectclasn;
int **currenttable, **globaltable;

void validate_init_var_data () {
  varval = new INTTYPE *[typetable.size];
  for ( int i = 0; i < typetable.size; i++ )
    if ( maxnvar[i] )
      varval[i] = new INTTYPE[maxnvar[i]];
    else
      varval[i] = NULL;
}

void validate_remove_var_data () {
  for ( int i = 0; i < typetable.size; i++ )
    if ( varval[i] )
      delete[] varval[i];
  delete[] varval;
}

void validate_init_history () {
  nodehistory = new Node*[treedepth];
  nodehistorysize = 0;
}

void validate_init_table ( int **&table ) {
  int cn = modetable.keynr;
  table = new int*[cn];
  for ( int i = 0; i < cn; i++ ) {
    table[i] = new int[cn];
    for ( int j = 0; j < cn; j++ )
      table[i][j] = 0;
  }
}

void validate_remove_history () {
  delete[] nodehistory;
}

void validate_remove_table ( int **&table ) {
  int cn = modetable.keynr;
  for ( int i = 0; i < cn; i++ )
    delete[] table[i];
  delete [] table;
  table = NULL;
}

void validate_clear_history () {
  nodehistorysize = 0;
}

void validate_push_to_history ( Node *node ) {
  nodehistory[nodehistorysize++] = node;
}

// in fact does not use node
void validate_pop_from_history ( Node *node ) {
  nodehistorysize--;
}

void validateInit () {
  validate_init_var_data ();
  validate_init_history ();
  correctclasn = incorrectclasn = 0;
  if ( validationtable )
    validate_init_table ( currenttable );
}

void validateRemove () {
  validate_remove_var_data ();
  validate_remove_history ();
  if ( validationtable )
    validate_remove_table ( currenttable );
}

void validatePrintTable ( int **table ) {
  int cn = modetable.keynr;

  for ( int i = 0; i  < cn; i++ ) {
    for ( int j = 0; j < cn; j++ )
      printf ( "%3i ", table[i][j] );
    printf ( "\n" );
  }
}

void validate_copy_key_var_data ( INTTYPE *value ) {
  int *params = akeypredicate->params, size = akeypredicate->size;
  int varn[typetable.size], i;

  for ( i = 0; i < typetable.size; i++ )
    varn[i] = 0;

  for ( i = 0; i < size; i++, params++ )
    varval[*params][varn[*params]++] = value[i];
}

void validate_evaluate_node ( Node *node, bool &result, ModeFactCoors *&resultcoors ) {
  ModeData *md = modetable.array + MODE ( node );
  int i, j, size = md->size;
  INTTYPE coors[md->size - md->nroutput];
  ModeParamData *mpd = md->params;
  INTTYPE *npd = node->params;

  for ( i = 0, j = 0; i < size; i++, mpd++, npd++ )
    if ( mpd->type == Constant )
      coors[j++] = mpd->number;
    else
      if ( mpd->type == Plus )
        coors[j++] = varval[mpd->number][*npd];

  md->table.table->setCurrentNode ( node );
  if ( md->outputs ) {
    resultcoors = md->table.modefacttable->getTrueFacts ( coors );
    result = resultcoors->empty ();
  }
  else {
    resultcoors = NULL;
    result = md->table.facttable->getFact ( coors );
  }
}

// value contains output values for the given node
void validate_copy_node_var_data ( Node *node, INTTYPE *value ) {
  ModeData *md = modetable.array + MODE ( node );
  PredicateData *pd = predicatetable.array[md->predicate];
  int outputnr = md->nroutput, *outputvarpos = md->outputvarpos, i;
  int *ppd = pd->params;
  INTTYPE *npd = node->params;

  for ( i = 0; i < outputnr; i++, outputvarpos++ )
    varval[ppd[*outputvarpos]][npd[*outputvarpos]] = value[i];
}

// return false: can not be proved
// return true: history can be proved
bool validate_evaluate_history ( int depth = 0 ) {
  ModeFactCoors *modefactcoors;
  bool result;
  Node *node = nodehistory[depth];
  INTTYPE *value;
  
  validate_evaluate_node ( node, result, modefactcoors );
  
  if ( depth == nodehistorysize - 1 ) {
    if ( modefactcoors )
      delete modefactcoors;
    return result;
  }

  if ( modefactcoors ) {
    while ( value = modefactcoors->value () ) {
      validate_copy_node_var_data ( node, value );
      if ( validate_evaluate_history ( depth + 1 ) ) {
        delete modefactcoors;
	return true;
      }
    }
    delete modefactcoors;
  }
  else
    if ( result )
      return validate_evaluate_history ( depth + 1 );
  
  return false;
}

void validateValue ( int cnr, void *additionaldata_class, INTTYPE *value, void *additionaldata_value, void *extraparams ) {
  Node *currentnode = root;
  bool result;
  validate_copy_key_var_data ( value );
  int clas;

  while ( currentnode ) {
#ifdef _DEBUG
    cout << currentnode->toString () << " ";
#endif
    validate_push_to_history ( currentnode );
    result = validate_evaluate_history ();
    if ( result ) {
#ifdef _DEBUG
      cout << "yes ";
#endif
      clas = currentnode->leftclass;
      currentnode = currentnode->left;
    }
    else {
#ifdef _DEBUG
      cout << "no ";
#endif
      validate_pop_from_history ( currentnode );
      clas = currentnode->rightclass;
      currentnode = currentnode->right;
    }
  }
  
  validate_clear_history ();

#ifdef _DEBUG
  cout << " " << *value << " is " << clas << " must be " << cnr << endl;
#endif
  
  if ( validationtable ) {
    currenttable[clas][cnr]++;
    globaltable[clas][cnr]++;
  }
  if ( clas != cnr )
    incorrectclasn++;
  else
    correctclasn++;
}

double validateTree () {
  if ( !root )
    return 0.0;
  //printTree ();
  validateInit ();
  test_keyfacttables->iterate ( validateValue );
  validateRemove ();

  return ( (double) correctclasn ) / (double) ( correctclasn + incorrectclasn ) * 100;
}

void validateInitValidationTable () {
  validate_init_table ( globaltable );
}

void validateRemoveValidationTable () {
  validatePrintTable ( globaltable );
  cout << "\\/ is   --> must be " << endl;
  validate_remove_table ( globaltable );
}