/***************************************************************************
                          qgraphatom.cpp  -  Gilder source
                             -------------------
    begin                : Sun Sep 24 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qgraphatom.h"
#include "qgraphtree.h"
#include <qfontmetrics.h>
#include <qpainter.h>
#include "gilder/globaldata.h"
#define DISTANCE 5

QGraphAtom::QGraphAtom(Node *node,QWidget *parent, const char *name ) : QWidget(parent,name) {
  this->node = node;
  highlight = false;
  setBackgroundColor ( white );
  fillCoors ();
  resize ( w, h );
}

QGraphAtom::QGraphAtom(char clas,QWidget *parent, const char *name ) : QWidget(parent,name) {
  this->node = NULL;
  this->clas = clas;
  highlight = false;
  setBackgroundColor ( white );

  QFontMetrics qfm = fontMetrics ();
  QRect iqr = qfm.boundingRect ( predicatetable.array[keynrtopnr[clas]]->name );
  h = iqr.height () + DISTANCE * 3;
  w = iqr.width () + DISTANCE * 2;

  resize ( w, h );
}

QGraphAtom::~QGraphAtom() {
  if ( node )
    delete[] x;
}

#define DISTANCE 5

void QGraphAtom::fillCoors () {
  QFontMetrics qfm = fontMetrics ();
  int mode = node->mode;
  ModeData *md = modetable.array + mode;
  int predicate = md->predicate;
  PredicateData *pd = predicatetable.array[predicate];
  int tx = DISTANCE;
  QString text;
  text = pd->name;
  text.append ( " " );
  md->table.table->setCurrentNode ( node );
  text.append ( md->table.table->string () );
  QRect iqr = qfm.boundingRect ( text );
  h = iqr.height () + DISTANCE * 3;
  int i;

  x = new int[pd->size + 2];
  x[0] = tx;
  tx += DISTANCE * 2 + iqr.width ();

  for ( i = 0; i < pd->size; i++ ) {
    x[i + 1] = tx;
    if ( md->params[i].type == Constant )
      text = typetable.getConstantName ( pd->params[i], md->params[i].number );
    else {
      text = typetable.getTypeName ( pd->params[i] );
      text.append ( "." );
      text.append ( QString ().setNum ( node->params[i] ) );
    }
    tx += qfm.boundingRect ( text ).width () + DISTANCE * 2;
  }

  w = x[i + 1] = tx - DISTANCE;
}

void QGraphAtom::paintEvent ( QPaintEvent * ) {
  QString name;
  int mode, x1, x2;
  ModeData *md;
  int predicate;
  PredicateData *pd;
  if ( node ) {
    mode = node->mode;
    md = modetable.array + mode;
    predicate = md->predicate;
    pd = predicatetable.array[predicate];
    name = pd->name;
    name.append ( " " );
    md->table.table->setCurrentNode ( node );
    name.append ( md->table.table->string () );
    x1 = x[0];
    x2 = x[1] - x1;
  }
  else {
    x1 = DISTANCE;
    x2 = w - 2 * DISTANCE;
    name = predicatetable.array[keynrtopnr[clas]]->name;
  }
  QString text;

  QPainter painter ( this );
  if ( highlight )
    painter.setPen ( QPen ( blue, 2 ) );
  else
    painter.setPen ( QPen ( black, 2 ) );
  painter.drawRect ( 1, 1, w - 1, h - 1 );
  painter.drawText ( x1, DISTANCE, x2, h, 0, name );

  if ( node ) {
    painter.drawLine ( x[1] - DISTANCE, 0, x[1] - DISTANCE, h );

    if ( highlight )
      painter.setPen ( QPen ( blue, 1 ) );
    else
      painter.setPen ( QPen ( black, 1 ) );

    for ( int i = 0; i < pd->size; i++ ) {
      if ( md->params[i].type == Constant )
        text = typetable.getConstantName ( pd->params[i], md->params[i].number );
      else {
        text = typetable.getTypeName ( pd->params[i] );
        text.append ( "." );
        text.append ( QString ().setNum ( node->params[i] ) );
      }
      painter.drawText ( x[i + 1], DISTANCE, x[i + 2], h, 0, text );
      painter.drawLine ( x[i + 1] - DISTANCE, 0, x[i + 1] - DISTANCE, h );
    }
  }
}

void QGraphAtom::mouseMoveEvent ( QMouseEvent * qme ) {
  QWidget *par1 = parentWidget ();
  QScrollView *par2 = (QScrollView*) par1->parentWidget ();
  QPainter p ( par1 );
  p.setRasterOp ( XorROP );
  p.setPen ( QPen ( darkGray, 1 ) );
  p.drawRect ( dragx, dragy, width (), height () );
  dragx = par2->childX ( this ) + qme->x () - tx;
  dragy = par2->childY ( this ) + qme->y () - ty;
  p.drawRect ( dragx, dragy, width (), height () );
  p.end ();
}

void QGraphAtom::mousePressEvent ( QMouseEvent *qme ) {
  QWidget *par1 = parentWidget ();
  QScrollView *par2 = (QScrollView*) par1->parentWidget ();

  dragx = par2->childX ( this );
  dragy = par2->childY ( this );
  tx = qme->x ();
  ty = qme->y ();

  QPainter p ( par1 );
  p.setRasterOp ( XorROP );
  p.setPen ( QPen ( darkGray, 1 ) );
  p.drawRect ( dragx, dragy, width (), height () );
  p.end ();
}

void QGraphAtom::mouseReleaseEvent ( QMouseEvent *qme ) {
  QWidget *par1 = parentWidget ();
  QGraphTree *par2 = (QGraphTree*) par1->parentWidget ();
  QPainter p ( par1 );
  p.setRasterOp ( XorROP );
  p.setPen ( QPen ( darkGray, 1 ) );
  p.drawRect ( dragx, dragy, width (), height () );
  p.end ();

  par2->moveChild ( this, dragx, dragy );
  par2->resizeContents ( par2->w, par2->h );
}

void QGraphAtom::setHighlight ( bool val ) {
  highlight = val;
  repaint ();
}