/***************************************************************************
                          qgraphtree.cpp  -  Gilder source
                             -------------------
    begin                : Sun Sep 24 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "gilder/globaldata.h"
#include "qgraphtree.h"
#include "gilder/output.h"
#define VERTINTERBOXD 75
#define HORZINTERBOXD 75
#define CIRCLESIZE 5

QGraphTree::QGraphTree(QWidget *parent, const char *name ) : QScrollView(parent,name) {
  qatoms.setAutoDelete ( true );
  qlines.setAutoDelete ( true );
  root = NULL;
}

QGraphTree::~QGraphTree() {

}

QGraphAtom *QGraphTree::addNode ( char clas, int depth, int *maxx, QList<int> &il ) {
  QWidget *vp = viewport ();
  QGraphAtom *graphatom = new QGraphAtom ( clas, vp );
  qatoms.append ( graphatom );
  il.append ( new int ( depth ) );

  addChild ( graphatom, maxx[depth], ( depth + 1 ) * VERTINTERBOXD );
  maxx[depth] += graphatom->width () + HORZINTERBOXD;

  graphatom->show ();

  return graphatom;
}

QGraphAtom *QGraphTree::addNode ( Node *node, int depth, int *maxx, QList<int> &il ) {
  QWidget *vp = viewport ();
  QGraphAtom *graphatom = new QGraphAtom ( node, vp );
  QGraphAtomArrow *qgaa;

  qatoms.append ( graphatom );
  il.append ( new int ( depth ) );

  qgaa = new QGraphAtomArrow;
  qgaa->beginatom = graphatom;
  if ( node->left )
    qgaa->endatom = addNode ( node->left, depth + 1, maxx, il );
  else
    qgaa->endatom = addNode ( node->leftclass, depth + 1, maxx, il );
  qlines.append ( qgaa );

  qgaa = new QGraphAtomArrow;
  qgaa->beginatom = graphatom;
  if ( node->right )
    qgaa->endatom = addNode ( node->right, depth + 1, maxx, il );
  else
    qgaa->endatom = addNode ( node->rightclass, depth + 1, maxx, il );
  qlines.append ( qgaa );

  addChild ( graphatom, maxx[depth], ( depth + 1 ) * VERTINTERBOXD );
  maxx[depth] += graphatom->width () + HORZINTERBOXD;

  graphatom->show ();

  return graphatom;
}

void QGraphTree::setTree ( Node *root, int maxdepth ) {
  int maxx[maxdepth], i, maxstartx = 0;
  QGraphAtom *qga;
  QList<int> il;
  il.setAutoDelete ( true );
  qatoms.clear ();
  qlines.clear ();

  for ( i = 0; i < maxdepth; i++ )
    maxx[i] = 0;

  addNode ( root, 0, maxx, il );

  for ( i = 0; i < maxdepth; i++ )
    if ( maxx[i] > maxstartx )
      maxstartx = maxx[i];

  // center the atom displays on the view
  qatoms.first ();
  il.first ();
  while ( qga = qatoms.current () ) {
    int cx = childX ( qga ) + ( maxstartx - maxx[*il.current ()] ) / 2;
    addChild ( qga,
      cx,
      childY ( qga )
    );
    qatoms.next ();
    il.next ();
  }

  w = maxstartx;
  h = maxdepth * VERTINTERBOXD;
  resizeContents ( w, h ); // test values

  viewport ()->update ();
}

void QGraphTree::drawContentsOffset ( QPainter * p, int offsetx, int offsety, int clipx, int clipy, int clipw, int cliph ) {
  QGraphAtomArrow *qgaa;
  int bx, by, ex, ey;
  p->setPen ( QPen ( lightGray, 2 ) );
  p->setBrush ( lightGray );
  viewport ()->erase ();

  qgaa = qlines.first ();
  while ( qgaa ) {
    bx = childX ( qgaa->beginatom ) + qgaa->beginatom->accessX () - offsetx;
    by = childY ( qgaa->beginatom ) + qgaa->beginatom->height () - offsety;
    ex = childX ( qgaa->endatom ) + qgaa->endatom->accessX () - offsetx;
    ey = childY ( qgaa->endatom ) - offsety;
    p->drawLine ( bx, by, ex, ey );
    p->drawEllipse ( bx - CIRCLESIZE, by - CIRCLESIZE, CIRCLESIZE * 2, CIRCLESIZE * 2 );
    p->drawEllipse ( ex - CIRCLESIZE, ey - CIRCLESIZE, CIRCLESIZE * 2, CIRCLESIZE * 2 );
    qgaa = qlines.next ();
  }
}
