/***************************************************************************
                          visualgilderdoc.cpp  -  Gilder source
                             -------------------
    begin                : Sun Sep 24 16:13:12 CEST 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "gilder/parser.h"
#include "gilder/interpreter.h"
#include "gilder/extern.h"
#include "gilder/tree_global.h"
#include "gilder/output.h"
#include "gilder/globaldata.h"
#include <sys/time.h>
#include "visualgilderdoc.h"

VisualgilderDoc::VisualgilderDoc()
{
  modified = false;
  tdl.setAutoDelete ( true );
}

VisualgilderDoc::~VisualgilderDoc()
{
}

void VisualgilderDoc::newDoc()
{
}

bool VisualgilderDoc::save()
{
  return true;
}

bool VisualgilderDoc::saveAs(const QString &filename)
{
  return true;
}

bool VisualgilderDoc::load(const QString &filename)
{
  QMonitorDialog *monitor = new QMonitorDialog;
  monitor->show ();
  bool result;
  showconstantnames = true;
  tdl.clear ();

  result = parse ( filename, interpret, 'g' );

  ERR << "[Finished reading input]";
  ERR_F

  if ( !result ) {
    ERR << "-- There were errors, bailing out --" << endl;
    ERR_F
  }
  else {
    timeval tv1, tv2;                                                                 assertfilledbias ();
    ERR << "[Extern initialization]";
    ERR_F
    extern_init ();
    bool validate = qApp->argc () > 1 && !strcmp ( qApp->argv () [ 1 ], "-v" );
    if ( validate ) {
      double t = 0.0, v;
      int tt = 0;
      ERR << "[Starting validation]";
      ERR_F

      //keyfacttables->shuffle ();
      train_keyfacttables = keyfacttables;

      for ( int i = 0; i < crossvalidation; i++ ) {
        TreeData *td = new TreeData;
        test_keyfacttables = train_keyfacttables->extractSample ( crossvalidation );

        gettimeofday ( &tv1, NULL );

        createTree ();

        gettimeofday ( &tv2, NULL );
        tt += ( tv2.tv_sec - tv1.tv_sec ) * 1000 + ( tv2.tv_usec - tv1.tv_usec ) / 1000;

        v = validateTree ();
        td->accur = v;
        td->cnt = i + 1;
        td->root = root;
        td->maxdepth = treedepth;
        tdl.append ( td );
        //removeTree ();

        ERR << i << ": correctly classified: " << v << "%";
        ERR_F
        t += v;

        train_keyfacttables->append ( test_keyfacttables );
      }
      ERR << "Total correctly classified: " << t / crossvalidation << "%";
      ERR_F

      ERR << "[Average searchtime: " << tt / crossvalidation << "ms]";
      ERR_F
    }
    else {
      test_keyfacttables = train_keyfacttables = keyfacttables;
      ERR << "[Starting search]" << endl;
      ERR_F
      gettimeofday ( &tv1, NULL );

      createTree ();

      gettimeofday ( &tv2, NULL );
      ERR << "Testing recall: " << validateTree () << "% correct";
      ERR_F
      ERR << "[Finished building in " << ( tv2.tv_sec - tv1.tv_sec ) * 1000 + ( tv2.tv_usec - tv1.tv_usec ) / 1000  << "ms]";
      ERR_F
      //printTree ();
      //removeTree ();
    }
  }
  emit documentChanged();
  monitordialog->exec ();
  return true;
}

bool VisualgilderDoc::isModified() const
{
  return modified;
}
