/***************************************************************************
                          visualgilderview.cpp  -  Gilder source
                             -------------------
    begin                : Sun Sep 24 16:13:12 CEST 2000
    copyright            : (C) 2000 by Siegfried Nijssen
    email                : snijssen@cs.leidenuniv.nl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "gilder/tree_global.h"
#include "visualgilderview.h"

VisualgilderView::VisualgilderView(QWidget *parent, VisualgilderDoc *doc) : QSplitter(QSplitter::Vertical, parent)
{
  /** connect doc with the view*/
  this->doc = doc;
  connect(doc, SIGNAL(documentChanged()), this, SLOT(slotDocumentChanged()));
  graphtree = new QGraphTree ( this );
  listview = NULL;
}

VisualgilderView::~VisualgilderView()
{
  delete graphtree;
  if ( listview )
    delete listview;
}

void VisualgilderView::slotDocumentChanged()
{
  if ( doc->tdl.isEmpty () ) {
    if ( listview ) {
      delete listview;
      listview = NULL;
    }
    if ( root )
      graphtree->setTree ( root, treedepth + 1 );
    //removeTree ();
  }
  else {
    if ( !listview ) {
      listview = new QListView ( this );
      listview->setAllColumnsShowFocus ( true );
      listview->addColumn ( "Accuracy", 100 );
      listview->addColumn ( "Tree" );
      connect(listview,SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(slotSelectionChanged(QListViewItem*)));
    }
    else
      listview->clear ();

    doc->tdl.first ();
    TreeData *td;

    while ( td = doc->tdl.current () ) {
      addTree ( td );
      doc->tdl.next ();
    }
  }
}

VisualgilderView::treeListViewItem::treeListViewItem ( QListView *lv, TreeData *td ):
  QListViewItem ( lv ) {
  this->td = td;
}

const char *VisualgilderView::treeListViewItem::text ( int column ) const {
  static QString tmp;
  if ( column == 0 )
    return tmp.sprintf ( "%f\%", td->accur );
  else
    if ( column == 1 ) {
      tmp.sprintf ( "Tree %i", td->cnt );
      return tmp;
    }
    else
      return NULL;
}

void VisualgilderView::addTree ( TreeData *td ) {
  new treeListViewItem ( listview, td );
}

void VisualgilderView::slotSelectionChanged ( QListViewItem *qlvi ) {
  treeListViewItem *qvi = (treeListViewItem*) qlvi;

  graphtree->setTree ( qvi->td->root, qvi->td->maxdepth );
}
